#include "Karte.h"
#include <Text.h>

// Inhalt der Karte Klasse aus Karte.h
// Konstruktor
Karte::Karte()
{
    gre = Vec2< int >( 0, 0 );
    spielerAnzahl = 0;
    teamAnzahl = 0;
    spielerFarbe = new Array< int >();
    teamFarbe = new Array< int >();
    teamName = new RCArray< Text >();
    teamGre = new Array< int >();
    ressources = new RCArray< RessourceStr >();
    objekte = new RCArray< ObjektStr >();
    spieler = new RCArray< SpielerStr >();
    teams = new RCArray< TeamStr >();
    ref = 1;
}

// Destruktor
Karte::~Karte()
{
    spielerFarbe->release();
    teamFarbe->release();
    teamName->release();
    teamGre->release();
    ressources->release();
    objekte->release();
    spieler->release();
    teams->release();
}

// nicht constant
void Karte::ladeSts( Datei *zDatei )
{
    zDatei->lese( (char*)&spielerAnzahl, 4 );
    zDatei->lese( (char*)&teamAnzahl, 4 );
    spielerFarbe->leeren();
    for( int i = 0; i < spielerAnzahl; i++ )
    {
        int farbe = 0;
        zDatei->lese( (char*)&farbe, 4 );
        spielerFarbe->set( farbe, i );
    }
    teamFarbe->leeren();
    for( int i = 0; i < teamAnzahl; i++ )
    {
        int farbe = 0;
        zDatei->lese( (char*)&farbe, 4 );
        teamFarbe->set( farbe, i );
    }
    teamName->leeren();
    for( int i = 0; i < teamAnzahl; i++ )
    {
        char ln = 0;
        zDatei->lese( &ln, 1 );
        char *txt = new char[ ln + 1 ];
        txt[ (int)ln ] = 0;
        zDatei->lese( txt, ln );
        Text *name = new Text( txt );
        delete[] txt;
        teamName->set( name, i );
    }
    teamGre->leeren();
    for( int i = 0; i < teamAnzahl; i++ )
    {
        int gre = 0;
        zDatei->lese( (char*)&gre, 4 );
        teamGre->set( gre, i );
    }
}

void Karte::ladeMap( Datei *zDatei, int mapId, KSGClient::SpielServerClient *zSpielKlient, Text *zGamePath )
{
    int ressourceAnz = 0;
    zDatei->lese( (char*)&ressourceAnz, 4 );
    for( int i = 0; i < ressourceAnz; i++ )
    {
        ressources->add( new RessourceStr( zDatei, mapId, zGamePath ) );
        zSpielKlient->setLadenProzent( 15 + (int)( 35 * ( (double)( i + 1 ) / ressourceAnz ) + 0.5 ) );
    }
    zDatei->lese( (char*)&gre.x, 4 );
    zDatei->lese( (char*)&gre.y, 4 );
    int objektAnzahl = 0;
    zDatei->lese( (char*)&objektAnzahl, 4 );
    for( int i = 0; i < objektAnzahl; i++ )
    {
        objekte->add( new ObjektStr( zDatei ) );
        zSpielKlient->setLadenProzent( 50 + (int)( 10 * ( (double)( i + 1 ) / objektAnzahl ) + 0.5 ) );
    }
    for( int i = 0; i < spielerAnzahl; i++ )
    {
        spieler->add( new SpielerStr( zDatei ) );
        zSpielKlient->setLadenProzent( 60 + (int)( 5 * ( (double)( i + 1 ) / spielerAnzahl ) + 0.5 ) );
    }
    for( int i = 0; i < teamAnzahl; i++ )
    {
        teams->add( new TeamStr( zDatei ) );
        zSpielKlient->setLadenProzent( 65 + (int)( 5 * ( (double)( i + 1 ) / teamAnzahl ) + 0.5 ) );
    }
}

// constant
Vec2< int > Karte::getSize() const
{
    return gre;
}

Spieler *Karte::createSpieler( int sNum, Schrift *zSchrift, KSGClient::InformationServerClient *zInfoK, Animation2DData *zFStart, Animation2DData *zFBurn ) const
{
    if( sNum < 0 || sNum >= spielerAnzahl )
        return 0;
    SpielerStr *player = 0;
    for( auto s = spieler->getIterator(); s; s++ )
    {
        if( s->id == sNum + 1 )
            player = s;
    }
    if( !player )
        return 0;
    Spieler *ret = new Spieler( zInfoK, zSchrift, player );
    for( auto i = ressources->getIterator(); i && i && i._; i++ )
    {
        if( i->getId() == player->m2d )
            ret->setModelData( i->zReccourceM2()->getThis(), zFStart, zFBurn );
        if( i->getId() == player->bild )
            ret->setSpielerFarbe( spielerFarbe->hat( sNum ) ? spielerFarbe->get( sNum ) : 0, i->zReccourceTxt() );
    }
    return ret;
}

Team *Karte::createTeam( int tNum ) const
{
    if( tNum < 0 || tNum >= teamAnzahl )
        return 0;
    TeamStr *team = 0;
    for( auto t = teams->getIterator(); t; t++ )
    {
        if( t->id == tNum + 1 )
            team = t;
    }
    if( !team )
        return 0;
    Team *ret = new Team( teamFarbe->get( tNum ), teamName->z( tNum )->getText(), team );
    return ret;
}

SpielObjekt *Karte::createObjekt( int oNum, Schrift *zSchrift ) const
{
    if( oNum < 0 || oNum > objekte->getEintragAnzahl() )
        return 0;
    ObjektStr *obj = 0;
    for( auto o = objekte->getIterator(); o; o++ )
    {
        if( o->id == oNum + 1 )
            obj = o;
    }
    if( !obj )
        return 0;
    SpielObjekt *ret = new SpielObjekt( obj, zSchrift );
    for( auto i = ressources->getIterator(); i && i._; i++ )
    {
        if( i->getId() == obj->m2d )
            ret->setModel( i->zReccourceM2()->getThis() );
        if( i->getId() == obj->bild )
            ret->setTextur( i->zReccourceTxt()->getThis() );
    }
    return ret;
}

int Karte::getMaxTeamAnzahl() const
{
    return teamAnzahl;
}

int Karte::getMaxSpielerAnzahl() const
{
    return spielerAnzahl;
}

int Karte::getZeichnungAnzahl() const
{
    return objekte->getEintragAnzahl();
}

int Karte::getTeamNummer( int sNum ) const
{
    int min = teamGre->get( 0 );
    for( int i = 1; i <= teamAnzahl; i++ )
    {
        if( sNum < min )
            return i - 1;
        min += teamGre->hat( i ) ? teamGre->get( i ) : 0;
    }
    return -1;
}

// Reference Counting
Karte *Karte::getThis()
{
    ref++;
    return this;
}

Karte *Karte::release()
{
    ref--;
    if( !ref )
        delete this;
    return 0;
}